# images in R

# install library
#install.packages("png")
#install.packages("jpeg")
#library(kernlab)
library(png)
library(jpeg)

# rotate the image 90 degrees...
transpose <- function( img )
{
  # split the image
  HT <- dim(img)[1]
  WD <- dim(img)[2]
  CH <- dim(img)[3]  # n channels
  r <- img[,,1]
  g <- img[,,2]
  b <- img[,,3]
  rt <- t(r)
  gt <- t(g)
  bt <- t(b)
  img_tr <- array(data=NA, dim=c(WD,HT,3))
  img_tr[,HT:1,1] <- rt  # the HT:1 flips the image left<->right 
  img_tr[,HT:1,2] <- gt
  img_tr[,HT:1,3] <- bt
  return(img_tr)
}

# may need to use getwd() and setwd("dir") to see the image
ar <- readJPEG("zach10p.jpg")
sb <- readJPEG("sbe.jpg")
# ar is an array of pixels, i.e., an image
plot.new()
# we rotate the image so that it looks correct
art <- transpose(ar)
# note that this might seem to imply "art"
# but it's far from art!
HT <- dim(art)[1]  # we'll use these globals...
WD <- dim(art)[2]  # and for width...
CH <- dim(art)[3]  # n channels
# may or may not need interpolate=F
# see ?readPNG for more image handling
rasterImage( art, 0, 0, 1, 1,  interpolate=F )

# crop so that it's a true green-screen
art <- art[1:270,,]
rasterImage( art, 0, 0, 1, 1,  interpolate=F )

# function to get a single-channel as a grayscale image
get_ch <- function( img, ch_to_show="rgb" )
{
  # compute all 6 channels and show the desired one...
  r <- img[,,1]
  g <- img[,,2]
  b <- img[,,3]
  
  # convert these to single rows (which rgb2hsv needs)
  red_row <- matrix(r,nrow=1)
  gre_row <- matrix(g,nrow=1)
  blu_row <- matrix(b,nrow=1)
  
  # hsv
  hsv <- rgb2hsv( red_row, gre_row, blu_row, maxColorValue=1.0 )
  
  hue_row <- hsv[1,]
  sat_row <- hsv[2,]
  val_row <- hsv[3,]
  
  HT <- dim(img)[1]
  WD <- dim(img)[2]
  
  h <- matrix(hue_row,nrow=HT,ncol=WD)
  s <- matrix(sat_row,nrow=HT,ncol=WD)
  v <- matrix(val_row,nrow=HT,ncol=WD)
  
  img_to_show <- img
  if (ch_to_show=="rgb") {
    ; # nothing to change
  } else if (ch_to_show=="r") {
    img_to_show <- r
  } else if (ch_to_show=="g") {
    img_to_show <- g
  } else if (ch_to_show=="b") {
    img_to_show <- b
  } else if (ch_to_show=="h") {
    img_to_show <- h
  } else if (ch_to_show=="s") {
    img_to_show <- s
  } else if (ch_to_show=="v") {
    img_to_show <- v
  } else if (ch_to_show=="hsv") {
    img_to_show[,,1] <- h
    img_to_show[,,2] <- s
    img_to_show[,,3] <- v
  }
  return(img_to_show)
}

# par(ask=T)


# needs for the image to be drawn 0,0,1,1
get_rgb <- function( img, npix=1, outof=1.0 )
{
  HT <- dim(img)[1]  # we'll use these globals...
  WD <- dim(img)[2]  # and for width...
  CH <- dim(img)[3]  # n channels
  
  mouse_coordinate_list <- locator( npix ) # gets npix clicks
  #print(mouse_coordinate_list)
  x <- 1+floor(mouse_coordinate_list$x * WD)
  y <- HT-floor(mouse_coordinate_list$y * HT)
  r <- vector("numeric",npix)
  g <- vector("numeric",npix)
  b <- vector("numeric",npix)
  for(i in 1:npix)
  {
    r[i] <- img[y[i],x[i],1]
    g[i] <- img[y[i],x[i],2]
    b[i] <- img[y[i],x[i],3]
  }
  hsv = rgb2hsv( r,g,b, maxColorValue=1.0 )
  h <- hsv[1,]
  s <- hsv[2,]
  v <- hsv[3,]
  if (outof == 255)
  {
    f <- function(x) { floor(256*x) }
    r <- f(r)
    g <- f(g)
    b <- f(b)
    h <- f(h)
    s <- f(s)
    v <- f(v)
  }
  return(data.frame(x,y,r,g,b))
}


# needs for the image to be drawn 0,0,1,1
get_all <- function( img, npix=1, outof=1.0 )
{
  HT <- dim(img)[1]  # we'll use these globals...
  WD <- dim(img)[2]  # and for width...
  CH <- dim(img)[3]  # n channels
  
  mouse_coordinate_list <- locator( npix ) # gets npix clicks
  #print(mouse_coordinate_list)
  x <- 1+floor(mouse_coordinate_list$x * WD)
  y <- HT-floor(mouse_coordinate_list$y * HT)
  r <- vector("numeric",npix)
  g <- vector("numeric",npix)
  b <- vector("numeric",npix)
  for(i in 1:npix)
  {
    r[i] <- img[y[i],x[i],1]
    g[i] <- img[y[i],x[i],2]
    b[i] <- img[y[i],x[i],3]
  }
  hsv = rgb2hsv( r,g,b, maxColorValue=1.0 )
  h <- hsv[1,]
  s <- hsv[2,]
  v <- hsv[3,]
  if (outof == 255)
  {
    f <- function(x) { floor(256*x) }
    r <- f(r)
    g <- f(g)
    b <- f(b)
    h <- f(h)
    s <- f(s)
    v <- f(v)
  }
  return(data.frame(x,y,r,g,b,h,s,v))
}


# function to get a single-channel as a grayscale image
change_ch <- function( img )
{
  # compute all 6 channels and show the desired one...
  r <- img[,,1]
  g <- img[,,2]
  b <- img[,,3]
  
  # convert these to single rows (which rgb2hsv needs)
  red_row <- matrix(r,nrow=1)
  gre_row <- matrix(g,nrow=1)
  blu_row <- matrix(b,nrow=1)
  
  # hsv
  hsv <- rgb2hsv( red_row, gre_row, blu_row, maxColorValue=1.0 )
  
  hue_row <- hsv[1,]
  sat_row <- hsv[2,]
  val_row <- hsv[3,]
  
  # data frame
  red <- t(red_row)
  gre <- t(gre_row)
  blu <- t(blu_row)
  df <- data.frame(red,gre,blu,hue_row,sat_row,val_row)
  
  HT <- dim(img)[1]
  WD <- dim(img)[2]
  
  # this is the test for green!
  GREEN <- (df$hue_row > .15)  # but it's definitely not complete!!
  
  df[which(GREEN),1] <- 1.0
  df[which(GREEN),2] <- 0.0
  df[which(GREEN),3] <- 0.0
  
  cat("len of whichGREEN ", length(which(GREEN)))
  
  new_r <- matrix(df$red,nrow=HT,ncol=WD)
  new_g <- matrix(df$gre,nrow=HT,ncol=WD)
  new_b <- matrix(df$blu,nrow=HT,ncol=WD)
  
  img_to_show <- img
  img_to_show[,,1] <- new_r
  img_to_show[,,2] <- new_g
  img_to_show[,,3] <- new_b
  
  return(img_to_show)
}



